/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 package de.bsvrz.vew.bmvew.bmvew.meldung;

 
/**
 * Die Klasse implementiert eine Zustandsmaschine  
 *
 * @version $Revision: 1.1 $ / $Date: 2009/09/24 12:50:45 $ / ($Author: Pittner $)
 *
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 *
 */
public class EndlicherAutomat {
     
    /**
     * Die Anzahl der Zustaende in der Zustandsmaschine
     */
    protected int anzahlZustaende = 0;
       
    /**
     * Der momentane Zustand
     */
    protected Zustand momentZustand = null;
     
    /**
     * Das Feld der Zustaende
     */
    protected Zustand[] eaZustaende = null;
    
    /**
     * Konstruktor
     */
    public EndlicherAutomat() {
        this(32);
    }
        
    /**
     * Konstruktor, mit max. Anzahl der Zustaende 
     * 
     * @param maxZustaende
     *          Anzahl der Erlaubten Zustände
     */
    public EndlicherAutomat(int maxZustaende) {
        eaZustaende = new Zustand[maxZustaende];
        //numStates = maxStates;
    }

    /**
     * Konstruktor, mit initialem Zustand
     *
     * @param initialZustand
     *          Anfangszustand der Zustandsmaschine
     */
    public EndlicherAutomat(Zustand initialZustand) {
        this();
        addZustand(initialZustand);
        setZustand(initialZustand);
    }

    /**
     * Konstruktor, mit max. Anzahl dr Zustände und initilem Zustand
     *
     * @param maxZustaende
     *          Anzahl der Erlaubten Zustände
     * @param initialZustand
     *          Anfangszustand der Zustandsmaschine
     */
    public EndlicherAutomat(int maxZustaende, Zustand initialZustand) {
        this(maxZustaende);
        addZustand(initialZustand);
        setZustand(initialZustand);
    }


    /**
     * Gibt den momentanen Zustandder Zustandsmaschine zurück
     * @return derzeitiger Zustand
     *
     */
    public Zustand getMomentZustand() {
        return momentZustand;
    }


    /**
     * Fügt einen Zustand hinzu
     *
     * @param zustand
     *          der Zustand der hinzugefuegt wird
     * @return
     *          Anzahl der derzeitigen Zustaende
     */
    public int addZustand(Zustand zustand) {

        if (anzahlZustaende == eaZustaende.length)
            throw new IllegalArgumentException("cannot add an additional state.");

        eaZustaende[anzahlZustaende++] = zustand;

        return anzahlZustaende;
    }


    /**
     * Setzt den Zustand der Zustandsmaschine
     *
     * @param zustand
     *          der Zustand
     */
    public void setZustand(Zustand zustand) {
        momentZustand = zustand;
    }


    /**
     * Fügt ein Feld von Zustaenden hinzu
     *
     * @param zustaende
     *          Feld der Zustaende
     * @return
     *          Antahl der Zustaende
     */
    public int addZustaende(Zustand[] zustaende) {
      anzahlZustaende = zustaende.length;
      eaZustaende = zustaende;

        return anzahlZustaende;
    }

    /**
     * Entfernt alle Zustaende
     */
    public void removeAlleZustaende() {
        // Keep the array allocated, but remove the
        // states.
        for (int i=0; i<eaZustaende.length; i++)
          eaZustaende[i] = null;
    }

    /**
     * Fügt einen Zustand zur Zustandsmaschine hinzu
     *
     * @param start
     *          der Anfangszustand
     * @param uebergang
     *          der Index des Übergangs
     * @param end
     *          der Endzustand
     */
    public void addUebergang(Zustand start, int uebergang, Zustand end) {
        start.addUebergang(uebergang, end);
    }

    /**
     * Realisert den Zustandsuebergang von einem Zustand zum anderen mit anhand des
     * uebergebenen Index. Die Methode erzeugt eine NullPointerException, wenn der
     * Übergang nicht für den Zustand definiert ist
     *
     * @param uebergang 
     *          der Index des Uebergangs
     * @return
     *          der neue Zustand der Zustandsmaschine
     */
    public Zustand uebergang(int uebergang) {
        Zustand neuZustand = momentZustand.uebergang(uebergang);
        
        if (neuZustand == null)
            throw new NullPointerException("transition not defined for this state.");
        
        momentZustand = neuZustand;
        
        return momentZustand;
    }
     
    /**
     * Holt den aktuellen Zustand
     * 
     * @param i 
     *          der Index des Zustands der geholt werden soll
     * @return
     *          Den Zustand des Index
     */
    public Zustand getZustand(int i) {
        return eaZustaende[i];
    }
    

    public int getAnzahlZustaende()
    {
      return anzahlZustaende;
    }

    public void setAnzahlZustaende(int anzahlZustaende)
    {
      this.anzahlZustaende = anzahlZustaende;
    }

 
 
    
}
